==> nmrium-0.53.0.tgz: package/lib/component/modal/editRange/forms/components/signal-tabs/SignalPeaksTable.js <==
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { v4 } from '@lukeed/uuid';
import { useFormikContext } from 'formik';
import { xFindClosestIndex } from 'ml-spectra-processing';
import { getShiftX } from 'nmr-processing';
import { useCallback, useMemo, useState } from 'react';
import { FaPlus, FaRegTrashAlt } from 'react-icons/fa';
import { Toolbar } from 'react-science/ui';
import { useChartData } from '../../../../../context/ChartContext';
import Button from '../../../../../elements/Button';
import ReactTable from '../../../../../elements/ReactTable/ReactTable';
import FormikInput from '../../../../../elements/formik/FormikInput';
import { usePanelPreferences } from '../../../../../hooks/usePanelPreferences';
import useSpectrum from '../../../../../hooks/useSpectrum';
import { useEvent } from '../../../../../utility/Events';
import { formatNumber } from '../../../../../utility/formatNumber';
const styles = {
    input: {
        width: '100%',
        padding: '0.25rem 0.5rem',
    },
    column: {
        padding: '2px',
    },
};
function getPeakKey(signalIndex, peakIndex, key) {
    const path = `signals[${signalIndex}].peaks.${peakIndex}`;
    if (!key) {
        return path;
    }
    return `${path}.${key}`;
}
export function SignalPeaksTable(props) {
    const { values, setFieldValue } = useFormikContext();
    const signal = values?.signals?.[values?.signalIndex] || {};
    const peaks = signal?.peaks || [];
    const delta = signal?.delta || 0;
    const spectrum = useSpectrum();
    const { data: { x: xArray, re }, } = spectrum;
    const shiftX = getShiftX(spectrum);
    const [lastSelectedPeakIndex, setLastSelectedPeakIndex] = useState(null);
    const { view: { spectra: { activeTab }, }, } = useChartData();
    const rangesPreferences = usePanelPreferences('ranges', activeTab);
    useEvent({
        onClick: (options) => {
            if (`${props.index}` === values.signalIndex &&
                typeof lastSelectedPeakIndex === 'number') {
                const delta = formatNumber(options.xPPM, rangesPreferences.deltaPPM.format);
                const xIndex = xFindClosestIndex(xArray, delta, { sorted: false });
                const intensity = formatNumber(re[xIndex], rangesPreferences.deltaPPM.format);
                void setFieldValue(getPeakKey(values?.signalIndex, lastSelectedPeakIndex), {
                    ...peaks[lastSelectedPeakIndex],
                    x: delta,
                    y: intensity,
                });
            }
        },
        onBrushEnd: (options) => {
            const { range: [from, to], } = options;
            if (`${props.index}` === values.signalIndex &&
                typeof lastSelectedPeakIndex === 'number') {
                const value = Number(formatNumber((to - from) / 2 + from, rangesPreferences.deltaPPM.format));
                void setFieldValue(getPeakKey(values.signalIndex, lastSelectedPeakIndex, 'x'), value);
            }
        },
    });
    const addHandler = useCallback((data) => {
        const xIndex = xFindClosestIndex(xArray, delta, { sorted: false });
        const peak = {
            id: v4(),
            x: delta,
            y: xIndex !== -1 ? re[xIndex] : 0,
            originalX: delta - shiftX,
            width: 1,
        };
        void setFieldValue(`signals[${values?.signalIndex}].peaks`, [
            ...data,
            peak,
        ]);
        setLastSelectedPeakIndex(data?.length || 0);
    }, [delta, re, setFieldValue, shiftX, values?.signalIndex, xArray]);
    const deleteHandler = useCallback((data, index) => {
        const peaks = data.filter((_, columnIndex) => columnIndex !== index);
        void setFieldValue(`signals[${values?.signalIndex}].peaks`, peaks);
        if (lastSelectedPeakIndex === index) {
            setLastSelectedPeakIndex(null);
        }
    }, [lastSelectedPeakIndex, setFieldValue, values?.signalIndex]);
    function deleteAllHandler() {
        void setFieldValue(`signals[${values?.signalIndex}].peaks`, []);
        setLastSelectedPeakIndex(null);
    }
    const changeDeltaHandler = useCallback((event, index) => {
        const delta = Number(event.target.value);
        const xIndex = xFindClosestIndex(xArray, delta, { sorted: false });
        void setFieldValue(`signals[${values?.signalIndex}].peaks.${index}.y`, re[xIndex]);
    }, [re, setFieldValue, values?.signalIndex, xArray]);
    const COLUMNS = useMemo(() => [
        {
            Header: '#',
            style: { width: '25px', ...styles.column },
            accessor: (_, index) => index + 1,
        },
        {
            Header: 'delta',
            style: { padding: 0, ...styles.column },
            Cell: ({ row }) => {
                return (_jsx(FormikInput, { name: `signals.${values?.signalIndex}.peaks.${row.index}.x`, style: { input: styles.input }, onChange: (event) => changeDeltaHandler(event, row.index), type: "number", onClick: (e) => e.stopPropagation() }));
            },
        },
        {
            Header: 'Intensity',
            style: { padding: 0, ...styles.column },
            Cell: ({ row }) => {
                return (_jsx(FormikInput, { name: `signals.${values?.signalIndex}.peaks.${row.index}.y`, style: { input: styles.input }, readOnly: true }));
            },
        },
        {
            Header: '',
            style: { width: '70px', ...styles.column },
            id: 'action-button',
            Cell: ({ data, row }) => {
                const record = row.original;
                return (_jsx("div", { style: { display: 'flex', justifyContent: 'space-around' }, children: !record?.name && (_jsx(Button.Danger, { fill: "outline", onClick: (e) => {
                            e.stopPropagation();
                            deleteHandler(data, row.index);
                        }, children: _jsx(FaRegTrashAlt, {}) })) }));
            },
        },
    ], [changeDeltaHandler, deleteHandler, values?.signalIndex]);
    function selectRowHandler(index) {
        setLastSelectedPeakIndex((prevIndex) => prevIndex === index ? null : index);
    }
    function handleActiveRow(row) {
        return row?.index === lastSelectedPeakIndex;
    }
    return (_jsxs("div", { style: { display: 'flex', flexDirection: 'column' }, children: [_jsx("div", { style: { padding: '5px 0', display: 'flex' }, children: _jsxs(Toolbar, { children: [_jsx(Toolbar.Item, { icon: _jsx(FaPlus, {}), title: "Add a new peak", intent: "success", onClick: () => addHandler(peaks) }), _jsx(Toolbar.Item, { icon: _jsx(FaRegTrashAlt, {}), title: "Delete all peaks", intent: "danger", onClick: deleteAllHandler })] }) }), _jsx("div", { style: {
                    maxHeight: '300px',
                    overflow: 'auto',
                }, children: _jsx(ReactTable, { data: peaks, columns: COLUMNS, onClick: (e, rowData) => selectRowHandler(rowData.index), activeRow: handleActiveRow, emptyDataRowText: "No peaks" }) })] }));
}

==> nmrium-0.54.0.tgz: package/lib/component/modal/editRange/forms/components/signal-tabs/SignalPeaksTable.js <==
import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { v4 } from '@lukeed/uuid';
import { useFormikContext } from 'formik';
import { xFindClosestIndex } from 'ml-spectra-processing';
import { getShiftX } from 'nmr-processing';
import { useCallback, useMemo, useState } from 'react';
import { FaPlus, FaRegTrashAlt } from 'react-icons/fa';
import { Toolbar } from 'react-science/ui';
import { useChartData } from '../../../../../context/ChartContext';
import Button from '../../../../../elements/Button';
import ReactTable from '../../../../../elements/ReactTable/ReactTable';
import FormikInput from '../../../../../elements/formik/FormikInput';
import { usePanelPreferences } from '../../../../../hooks/usePanelPreferences';
import useSpectrum from '../../../../../hooks/useSpectrum';
import { useEvent } from '../../../../../utility/Events';
import { formatNumber } from '../../../../../utility/formatNumber';
const styles = {
    input: {
        width: '100%',
        padding: '0.25rem 0.5rem',
    },
    column: {
        padding: '2px',
    },
};
function getPeakKey(signalIndex, peakIndex, key) {
    const path = `signals[${signalIndex}].peaks.${peakIndex}`;
    if (!key) {
        return path;
    }
    return `${path}.${key}`;
}
export function SignalPeaksTable(props) {
    const { values, setFieldValue } = useFormikContext();
    const signal = values?.signals?.[values?.signalIndex] || {};
    const peaks = signal?.peaks || [];
    const delta = signal?.delta || 0;
    const spectrum = useSpectrum();
    const { data: { x: xArray, re }, } = spectrum;
    const shiftX = getShiftX(spectrum);
    const [lastSelectedPeakIndex, setLastSelectedPeakIndex] = useState(null);
    const { view: { spectra: { activeTab }, }, } = useChartData();
    const rangesPreferences = usePanelPreferences('ranges', activeTab);
    useEvent({
        onClick: (options) => {
            if (`${props.index}` === values.signalIndex &&
                typeof lastSelectedPeakIndex === 'number') {
                const delta = formatNumber(options.xPPM, rangesPreferences.deltaPPM.format);
                const xIndex = xFindClosestIndex(xArray, delta, { sorted: false });
                const intensity = formatNumber(re[xIndex], rangesPreferences.deltaPPM.format);
                void setFieldValue(getPeakKey(values?.signalIndex, lastSelectedPeakIndex), {
                    ...peaks[lastSelectedPeakIndex],
                    x: delta,
                    y: intensity,
                });
            }
        },
        onBrushEnd: (options) => {
            const { range: [from, to], } = options;
            if (`${props.index}` === values.signalIndex &&
                typeof lastSelectedPeakIndex === 'number') {
                const value = Number(formatNumber((to - from) / 2 + from, rangesPreferences.deltaPPM.format));
                void setFieldValue(getPeakKey(values.signalIndex, lastSelectedPeakIndex, 'x'), value);
            }
        },
    });
    const addHandler = useCallback((data) => {
        const xIndex = xFindClosestIndex(xArray, delta, { sorted: false });
        const peak = {
            id: v4(),
            x: delta,
            y: xIndex !== -1 ? re[xIndex] : 0,
            originalX: delta - shiftX,
            width: 1,
        };
        void setFieldValue(`signals[${values?.signalIndex}].peaks`, [
            ...data,
            peak,
        ]);
        setLastSelectedPeakIndex(data?.length || 0);
    }, [delta, re, setFieldValue, shiftX, values?.signalIndex, xArray]);
    const deleteHandler = useCallback((data, index) => {
        const peaks = data.filter((_, columnIndex) => columnIndex !== index);
        void setFieldValue(`signals[${values?.signalIndex}].peaks`, peaks);
        if (lastSelectedPeakIndex === index) {
            setLastSelectedPeakIndex(null);
        }
    }, [lastSelectedPeakIndex, setFieldValue, values?.signalIndex]);
    function deleteAllHandler() {
        void setFieldValue(`signals[${values?.signalIndex}].peaks`, []);
        setLastSelectedPeakIndex(null);
    }
    const changeDeltaHandler = useCallback((event, index) => {
        const delta = Number(event.target.value);
        const xIndex = xFindClosestIndex(xArray, delta, { sorted: false });
        void setFieldValue(`signals[${values?.signalIndex}].peaks.${index}.y`, re[xIndex]);
    }, [re, setFieldValue, values?.signalIndex, xArray]);
    const COLUMNS = useMemo(() => [
        {
            Header: '#',
            style: { width: '25px', ...styles.column },
            accessor: (_, index) => index + 1,
        },
        {
            Header: 'delta',
            style: { padding: 0, ...styles.column },
            Cell: ({ row }) => {
                return (_jsx(FormikInput, { name: `signals.${values?.signalIndex}.peaks.${row.index}.x`, style: { input: styles.input }, onChange: (event) => changeDeltaHandler(event, row.index), type: "number", onClick: (e) => e.stopPropagation() }));
            },
        },
        {
            Header: 'Intensity',
            style: { padding: 0, ...styles.column },
            Cell: ({ row }) => {
                return (_jsx(FormikInput, { name: `signals.${values?.signalIndex}.peaks.${row.index}.y`, style: { input: styles.input }, readOnly: true }));
            },
        },
        {
            Header: '',
            style: { width: '70px', ...styles.column },
            id: 'action-button',
            Cell: ({ data, row }) => {
                const record = row.original;
                return (_jsx("div", { style: { display: 'flex', justifyContent: 'space-around' }, children: !record?.name && (_jsx(Button.Danger, { fill: "outline", onClick: (e) => {
                            e.stopPropagation();
                            deleteHandler(data, row.index);
                        }, children: _jsx(FaRegTrashAlt, {}) })) }));
            },
        },
    ], [changeDeltaHandler, deleteHandler, values?.signalIndex]);
    function selectRowHandler(index) {
        setLastSelectedPeakIndex((prevIndex) => prevIndex === index ? null : index);
    }
    function handleActiveRow(row) {
        return row?.index === lastSelectedPeakIndex;
    }
    return (_jsxs("div", { style: { display: 'flex', flexDirection: 'column' }, children: [_jsx("div", { style: { padding: '5px 0', display: 'flex' }, children: _jsxs(Toolbar, { children: [_jsx(Toolbar.Item, { icon: _jsx(FaPlus, {}), tooltip: "Add a new peak", intent: "success", onClick: () => addHandler(peaks) }), _jsx(Toolbar.Item, { icon: _jsx(FaRegTrashAlt, {}), tooltip: "Delete all peaks", intent: "danger", onClick: deleteAllHandler })] }) }), _jsx("div", { style: {
                    maxHeight: '300px',
                    overflow: 'auto',
                }, children: _jsx(ReactTable, { data: peaks, columns: COLUMNS, onClick: (e, rowData) => selectRowHandler(rowData.index), activeRow: handleActiveRow, emptyDataRowText: "No peaks" }) })] }));
}
